@@ -23,9 +23,6 @@
 #ifdef WIN32
 #  include <shlobj.h>
 #endif
-#if HAVE_PAPER_H
-#include <paper.h>
-#endif
 #include <fontconfig/fontconfig.h>
 #include "goo/gmem.h"
 #include "goo/GooString.h"
@@ -324,24 +321,8 @@ GlobalParams::GlobalParams(char *cfgFileName) {
   cMapDirs = new GooHash(gTrue);
   toUnicodeDirs = new GooList();
   displayFonts = new GooHash();
-#if HAVE_PAPER_H
-  char *paperName;
-  const struct paper *paperType;
-  paperinit();
-  if ((paperName = systempapername())) {
-    paperType = paperinfo(paperName);
-    psPaperWidth = (int)paperpswidth(paperType);
-    psPaperHeight = (int)paperpsheight(paperType);
-  } else {
-    error(-1, "No paper information available - using defaults");
-    psPaperWidth = defPaperWidth;
-    psPaperHeight = defPaperHeight;
-  }
-  paperdone();
-#else
-  psPaperWidth = defPaperWidth;
-  psPaperHeight = defPaperHeight;
-#endif
+  psPaperWidth = -1;
+  psPaperHeight = -1;
   psImageableLLX = psImageableLLY = 0;
   psImageableURX = psPaperWidth;
   psImageableURY = psPaperHeight;
@@ -1122,7 +1103,7 @@ GBool findModifier(const char *name, const char *modifier, const char **start)
   }
 }
 
-FcPattern *buildFcPattern(GfxFont *font)
+static FcPattern *buildFcPattern(GfxFont *font)
 {
   int weight = FC_WEIGHT_NORMAL,
       slant = FC_SLANT_ROMAN,
@@ -1266,7 +1247,7 @@ FcPattern *buildFcPattern(GfxFont *font)
 
 DisplayFontParam *GlobalParams::getDisplayFont(GfxFont *font) {
   DisplayFontParam *dfp;
-  FcPattern *p=0,*m=0;
+  FcPattern *p=0;
 
   GooString *fontName = font->getName();
   if (!fontName) return NULL;
@@ -1278,38 +1259,44 @@ DisplayFontParam *GlobalParams::getDisplayFont(GfxFont *font) {
     FcChar8* s;
     char * ext;
     FcResult res;
+    FcFontSet *set;
+    int i;
     p = buildFcPattern(font);
 
     if (!p)
       goto fin;
     FcConfigSubstitute(FCcfg, p, FcMatchPattern);
     FcDefaultSubstitute(p);
-    m = FcFontMatch(FCcfg,p,&res);
-    if (!m)
-      goto fin; 
-    res = FcPatternGetString(m, FC_FILE, 0, &s);
-    if (res != FcResultMatch || !s)
-      goto fin; 
-    ext = strrchr((char*)s,'.');
-    if (!ext)
+    set = FcFontSort(FCcfg, p, FcFalse, NULL, &res);
+    if (!set)
       goto fin;
-    if (!strncasecmp(ext,".ttf",4))
+    for (i = 0; i < set->nfont; ++i)
     {
-      dfp = new DisplayFontParam(fontName->copy(), displayFontTT);  
-      dfp->tt.fileName = new GooString((char*)s);
+      res = FcPatternGetString(set->fonts[i], FC_FILE, 0, &s);
+      if (res != FcResultMatch || !s)
+        continue;
+      ext = strrchr((char*)s,'.');
+      if (!ext)
+        continue;
+      if (!strncasecmp(ext,".ttf",4) || !strncasecmp(ext, ".ttc", 4))
+      {
+        dfp = new DisplayFontParam(fontName->copy(), displayFontTT);  
+        dfp->tt.fileName = new GooString((char*)s);
+        FcPatternGetInteger(set->fonts[i], FC_INDEX, 0, &(dfp->tt.faceIndex));
+      }
+      else if (!strncasecmp(ext,".pfa",4) || !strncasecmp(ext,".pfb",4)) 
+      {
+        dfp = new DisplayFontParam(fontName->copy(), displayFontT1);  
+        dfp->t1.fileName = new GooString((char*)s);
+      }
+      else
+        continue;
+      displayFonts->add(dfp->name,dfp);
+      break;
     }
-    else if (!strncasecmp(ext,".pfa",4) || !strncasecmp(ext,".pfb",4)) 
-    {
-      dfp = new DisplayFontParam(fontName->copy(), displayFontT1);  
-      dfp->t1.fileName = new GooString((char*)s);
-    }
-    else
-      goto fin;
-    displayFonts->add(dfp->name,dfp);
+    FcFontSetDestroy(set);
   }
 fin:
-  if (m)
-    FcPatternDestroy(m);
   if (p)
     FcPatternDestroy(p);
 
