@@ -20,6 +20,7 @@
 #include "XRef.h"
 #include "Error.h"
 #include "Decrypt.h"
+#include "UGooString.h"
 
 Parser::Parser(XRef *xrefA, Lexer *lexerA) {
   xref = xrefA;
@@ -68,7 +69,7 @@ Object *Parser::getObj(Object *obj,
 
   // dictionary or stream
   } else if (buf1.isCmd("<<")) {
-    shift();
+    shift(objNum);
     obj->initDict(xref);
     while (!buf1.isCmd(">>") && !buf1.isEOF()) {
       if (!buf1.isName()) {
@@ -82,6 +83,7 @@ Object *Parser::getObj(Object *obj,
 	  break;
 	}
 	obj->dictAdd(key, getObj(&obj2, fileKey, keyLength, objNum, objGen));
+	gfree(key);
       }
     }
     if (buf1.isEOF())
@@ -192,7 +194,7 @@ Stream *Parser::makeStream(Object *dict) {
   return str;
 }
 
-void Parser::shift() {
+void Parser::shift(int objNum) {
   if (inlineImg > 0) {
     if (inlineImg < 2) {
       ++inlineImg;
@@ -210,5 +212,5 @@ void Parser::shift() {
   if (inlineImg > 0)		// don't buffer inline image data
     buf2.initNull();
   else
-    lexer->getObj(&buf2);
+    lexer->getObj(&buf2, objNum);
 }
