@@ -76,18 +76,29 @@ QImage Page::renderToImage(double xres, double yres) const
   color_ptr = bitmap->getDataPtr ();
   int bw = output_dev->getBitmap()->getWidth();
   int bh = output_dev->getBitmap()->getHeight();
-
-  QImage img( bw, bh, 32 );
-  SplashColorPtr pixel = new Guchar[4];
-  for (int i = 0; i < bw; i++)
+  SplashColorPtr dataPtr = output_dev->getBitmap()->getDataPtr();
+  
+  if (QImage::BigEndian == QImage::systemByteOrder())
   {
-    for (int j = 0; j < bh; j++)
+    uchar c;
+    int count = bw * bh * 4;
+    for (int k = 0; k < count; k += 4)
     {
-      output_dev->getBitmap()->getPixel(i, j, pixel);
-      img.setPixel( i, j, qRgb( pixel[0], pixel[1], pixel[2] ) );
+      c = dataPtr[k];
+      dataPtr[k] = dataPtr[k+3];
+      dataPtr[k+3] = c;
+
+      c = dataPtr[k+1];
+      dataPtr[k+1] = dataPtr[k+2];
+      dataPtr[k+2] = c;
     }
   }
-  delete[] pixel;
+  
+  // construct a qimage SHARING the raw bitmap data in memory
+  QImage img( dataPtr, bw, bh, 32, 0, 0, QImage::IgnoreEndian );
+  img = img.copy();
+  // unload underlying xpdf bitmap
+  output_dev->startPage( 0, NULL );
 
   return img;
 }
