@@ -24,8 +24,12 @@
 
 #include <QtCore/QByteArray>
 #include <QtCore/QDateTime>
+#include <QtCore/QVector>
 #include <QtGui/QPixmap>
+#include <QtXml/QDomDocument>
 
+#include <poppler-annotation.h>
+#include <poppler-link.h>
 #include <poppler-page-transition.h>
 
 class EmbFile;
@@ -51,6 +55,7 @@ namespace Poppler {
 	the text is found.
     */
     class TextBox {
+    friend class Page;
     public:
       /**
 	 The default constructor sets the text and the rectangle the
@@ -70,6 +75,12 @@ namespace Poppler {
       */
       const QRectF &boundingBox() const;
 
+      TextBox *nextWord() const;
+
+      double edge(int i) const;
+
+      bool hasSpaceAfter() const;
+
     private:
 	TextBoxData *m_data;
     };
@@ -83,21 +94,20 @@ namespace Poppler {
     class FontInfo {
     public:
 	enum Type {
-	    unknown,
-	    Type1,
-	    Type1C,
-	    Type3,
-	    TrueType,
-	    CIDType0,
-	    CIDType0C,
-	    CIDTrueType
+	    unknown = 0,
+	    Type1 = 1,
+	    Type1C = 2,
+	    Type3 = 3,
+	    TrueType = 4,
+	    CIDType0 = 5,
+	    CIDType0C = 6,
+	    CIDTrueType = 7
 	};
 	
 	/**
 	   Create a new font information container
 	*/
-	FontInfo( const QString &fontName, const bool isEmbedded,
-		  const bool isSubset, Type type);
+	FontInfo( const FontInfoData &fid );
 	
 	FontInfo( const FontInfo &fi );
 	
@@ -108,6 +118,11 @@ namespace Poppler {
 	*/
 	const QString &name() const;
 
+	/**
+	   The path of the font file used to represent this font on this system.
+	*/
+	const QString &file() const;
+
 	/**
 	   Whether the font is embedded in the file, or not
 
@@ -237,6 +252,8 @@ namespace Poppler {
 
 	   \param yres vertical resolution of the graphics device, in
 	   dots per inch
+	
+	   \param doLinks calculate links
 
 	   \warning The parameter (x,y,w,h) are not
 	   well-tested. Unusual or meaningless parameters may lead to
@@ -244,7 +261,7 @@ namespace Poppler {
 
 	   \returns a QImage of the page, or a null image on failure.
         */
-	QImage splashRenderToImage(double xres=72.0, double yres=72.0, int x=-1, int y=-1, int w=-1, int h=-1) const;
+	QImage splashRenderToImage(double xres=72.0, double yres=72.0, int x=-1, int y=-1, int w=-1, int h=-1, bool doLinks = false) const;
 	
 	/**
 	   Render the page to a QPixmap using the Splash renderer
@@ -267,7 +284,7 @@ if (pixmap == 0) {
 delete pixmap;
 @endcode
 	 */
-	QPixmap *splashRenderToPixmap(double xres=72.0, double yres=72.0, int x=-1, int y=-1, int w=-1, int h=-1) const;
+	QPixmap *splashRenderToPixmap(double xres=72.0, double yres=72.0, int x=-1, int y=-1, int w=-1, int h=-1, bool doLinks = false) const;
 
 	/**
 	    Render the page to a pixmap using the Arthur (Qt4) renderer
@@ -301,6 +318,22 @@ delete pixmap;
 	   If rect is null, all text on the page is given
 	**/
 	QString text(const QRectF &rect) const;
+	
+	
+	enum SearchDirection { FromTop, NextResult, PreviousResult };
+	enum SearchMode { CaseSensitive, CaseInsensitive };
+	
+	/**
+	   Returns true if the specified text was found.
+
+	   \param text the text the search
+	   \param rect in all directions is used to return where the text was found, for NextResult and PreviousResult
+	               indicates where to continue searching for
+	   \param direction in which direction do the search
+	   \param caseSensitive be case sensitive?
+	**/
+	bool search(const QString &text, QRectF &rect, SearchDirection direction, SearchMode caseSensitive) const;
+	
 
 	/**
 	   Returns a list of text of the page
@@ -352,6 +385,22 @@ delete pixmap;
 	   The orientation of the page
 	*/
 	Orientation orientation() const;
+	
+	/**
+	  The default CTM
+	*/
+	void defaultCTM(double *CTM, double dpiX, double dpiY, int rotate, bool upsideDown);
+	
+	/**
+	  Gets the links of the page once it has been rendered if doLinks was true
+	*/
+	QList<Link*> links() const;
+	
+	/**
+	 Returns the annotations of the page
+	*/
+	QList<Annotation*> annotations () const;
+	
     private:
 	Page(const Document *doc, int index);
 	PageData *m_page;
@@ -401,9 +450,9 @@ delete pixmap;
 	   \warning The application owns the pointer to Document, and this should
 	   be deleted when no longer required.
 	*/
-	static Document *Document::load(const QString & filePath,
-					const QByteArray &ownerPassword=QByteArray(),
-					const QByteArray &userPassword=QByteArray());
+	static Document *load(const QString & filePath,
+			      const QByteArray &ownerPassword=QByteArray(),
+			      const QByteArray &userPassword=QByteArray());
   
 	/**
 	   Get a specified page
@@ -611,12 +660,37 @@ delete pixmap;
 	   Whether there are any documents embedded in this PDF document.
 	*/
 	bool hasEmbeddedFiles() const;
+	
+	/**
+	  Gets the TOC of the Document, it is application responsabiliy to delete
+	  it when no longer needed
+	
+	  * In the tree the tag name is the 'screen' name of the entry. A tag can have
+	  * attributes. Here follows the list of tag attributes with meaning:
+	  * - Destination: A string description of the referred destination
+	  * - DestinationName: A 'named reference' to the viewport that must be converted
+	  *      using linkDestination( *destination_name* )
+	  * - ExternalFileName: A link to a external filename
+	
+	  Returns NULL if the Document does not have TOC
+	*/
+	QDomDocument *toc() const;
+	
+	LinkDestination *linkDestination( const QString &name );
+	
+	bool print(const QString &fileName, const QList<int> pageList, double hDPI, double vDPI, int rotate);
+	
+	/**
+	  Sets the paper color, if not set it is white by default.
+	 */
+	void setPaperColor(const QColor &color);
+	QColor paperColor() const;
 
-	Document::~Document();
+	~Document();
   
     private:
 	DocumentData *m_doc;
-	Document::Document(DocumentData *dataA);
+	Document(DocumentData *dataA);
     };
 
     /**
